@@ -66,6 +66,29 @@ static int rmrf(const char* path)
   return forkexeclp("rm", "rm", "-rf", path, NULL);
 }
 
+/**
+   Remove a directory, but only if nothing is mounted under it.
+
+   @return 0 on success
+ */
+static int rmrf_check_mount(const char* path)
+{
+  /*
+   * A directory bind-mounted into path will be cleaned out by rmrf().
+   * To avoid that potential disaster we want to detect if there's
+   * something mounted under path before deleting it.
+   */
+
+  char* canonical_path = strdupa(path);
+
+  canonicalize_doubleslash(path, canonical_path);
+  if (check_mountpoint(canonical_path)) {
+    return 1;
+  }
+
+  return rmrf(path);
+}
+
 /**
  * @return .ilist path as malloc()ed string, or NULL on error.
  */
@@ -180,7 +203,7 @@ static int cpbuilder_internal_cowcopy(const struct pbuilderconfig* pc)
   char *ilistfile;
 
   printf(" -> Copying COW directory\n");
-  if (0!=rmrf(pc->buildplace))
+  if (0!=rmrf_check_mount(pc->buildplace))
     return 1;
   if (0!=forkexeclp("cp", "cp", "-al", pc->basepath, pc->buildplace, NULL))
     return 1;
@@ -220,23 +243,9 @@ static int cpbuilder_internal_cowcopy(const struct pbuilderconfig* pc)
  */
 static int cpbuilder_internal_cleancow(const struct pbuilderconfig* pc)
 {
-  /*
-   * A directory bind-mounted into pc->buildplace, will be cleaned out
-   * by rmrf() To avoid that potential disaster we want to detect if
-   * there's something mounted under the chroot before running the
-   * cleanup procedure.
-   */
-
-  char* canonicalized_buildplace = strdupa(pc->buildplace);
-
   printf(" -> Cleaning COW directory\n");
 
-  canonicalize_doubleslash(pc->buildplace, canonicalized_buildplace);
-  if (check_mountpoint(canonicalized_buildplace)) {
-    return 1;
-  }
-
-  if (0!=rmrf(pc->buildplace))
+  if (0!=rmrf_check_mount(pc->buildplace))
     return 1;
   return 0;
 }
@@ -332,10 +341,10 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
       PBUILDER_ADD_PARAM("--buildresult");
       PBUILDER_ADD_PARAM(pc->buildresult);
     }
-  if (pc->debbuildopts)
+  if (pc->extrapackages)
     {
-      PBUILDER_ADD_PARAM("--debbuildopts");
-      PBUILDER_ADD_PARAM(pc->debbuildopts);
+      PBUILDER_ADD_PARAM("--extrapackages");
+      PBUILDER_ADD_PARAM(pc->extrapackages);
     }
   if (pc->binary_arch)
     {
@@ -381,14 +390,29 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
   PBUILDER_ADD_PARAM(pc->distribution);
   PBUILDER_ADD_PARAM("--no-targz");
   PBUILDER_ADD_PARAM("--extrapackages");
-  PBUILDER_ADD_PARAM("cowdancer");
+  if (pc->extrapackages)
+    {
+      char *extrapackages;
+      if (0>asprintf(&extrapackages, "%s cowdancer", pc->extrapackages))
+	{
+	  fprintf(stderr, "cowdancer: out of memory.\n");
+	  ret = 1;
+	  goto out;
+	}
+      PBUILDER_ADD_PARAM(extrapackages);
+    }
+  else
+    {
+      PBUILDER_ADD_PARAM("cowdancer");
+    }
   PBUILDER_ADD_PARAM(NULL);
   ret=forkexecvp(pbuildercommandline);
 
+out:
   if (ret)
     {
       printf("pbuilder create failed\n");
-      if (0!=rmrf(pc->basepath))
+      if (0!=rmrf_check_mount(pc->basepath))
 	{
 	  fprintf(stderr, "Could not remove original tree\n");
 	}
@@ -429,7 +453,7 @@ int cpbuilder_internal_saveupdate(const struct pbuilderconfig* pc)
       ret=-1;
       goto out;
     }
-  if (0!=rmrf(buf_tmpfile))
+  if (0!=rmrf_check_mount(buf_tmpfile))
     {
       printf("Could not remove original tree\n");
       ret=-1;
@@ -626,6 +650,11 @@ int cpbuilder_update(const struct pbuilderconfig* pc)
       PBUILDER_ADD_PARAM("--distribution");
       PBUILDER_ADD_PARAM(pc->distribution);
     }
+  if (pc->extrapackages)
+    {
+      PBUILDER_ADD_PARAM("--extrapackages");
+      PBUILDER_ADD_PARAM(pc->extrapackages);
+    }
   PBUILDER_ADD_PARAM("--no-targz");
   PBUILDER_ADD_PARAM("--internal-chrootexec");
   PBUILDER_ADD_PARAM(buf_chroot);
@@ -644,7 +673,7 @@ int cpbuilder_update(const struct pbuilderconfig* pc)
 	      fprintf(stderr, "E: could not update with cowdancer, try --no-cowdancer-update option\n");
 	    }
 
-	  if (0!=rmrf(pc->buildplace))
+	  if (0!=rmrf_check_mount(pc->buildplace))
 	    {
 	      fprintf(stderr, "Could not remove original tree\n");
 	    }
