@@ -181,6 +181,25 @@ static int copy_file_contents_through_temp(FILE* f,
   return ret;
 }
 
+static const char* format_for_image(const char* file)
+{
+  static const char *formats[][2] = {
+    { ".cowdev", "qcow2" },
+    { ".dev", "raw" },
+    { ".qemu", "raw" }
+  };
+  int file_len = strlen(file);
+  for (int i = 0; i < sizeof(formats) / sizeof(formats[0]); ++i)
+    {
+      const char *ext = formats[i][0];
+      const char *format = formats[i][1];
+      int ext_len = strlen(ext);
+      if (file_len >= ext_len && !strcmp(ext, file + file_len - ext_len))
+	return format;
+    }
+  return NULL;
+}
+
 /**
    run qemu until exit signal is received from within QEMU via serial
    console.
@@ -216,6 +235,13 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
       FD_ZERO(&readfds);
       while (1)
 	{
+	  int status;
+	  if (0<waitpid(child, &status, WNOHANG))
+	    {
+	      /* child has exited */
+	      printf("\n   -> qemu exited unexpectedly: %d\n", status);
+	      break;
+	    }
 	  FD_SET(sp[0],&readfds);
 	  if (-1!=(select(sp[0]+1,&readfds, NULL, NULL, NULL)))
 	    {
@@ -234,8 +260,6 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
 		  if ((matchptr=memmem(buf, count,
 				       qemu_keyword, strlen(qemu_keyword)))!=0)
 		    {
-		      int status;
-
 		      exit_code = atoi(matchptr + strlen(qemu_keyword));
 		      printf("\n  -> received termination message from inside qemu with exit-code %i, killing child process (qemu:%i)\n",
 			     exit_code,
@@ -268,6 +292,8 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
       /* this is the child process */
       const char* qemu = qemu_arch_qemu(pc->arch);
       const char* machine = qemu_arch_qemumachine(pc->arch);
+      const char* hda_format = format_for_image(hda);
+      const char* hdb_format = format_for_image(hdb);
       char* hda_command;
       char* hdb_command;
       char* append_command;
@@ -284,18 +310,37 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
 	exit(1);
       }
 
+      if (kernel_image == NULL || !strcmp(kernel_image, "")) {
+	fprintf(stderr, "No KERNEL_IMAGE defined in pbuilderrc\n");
+	exit(1);
+      }
+
       asprintf(&mem, "%i", pc->memory_megs);
 
+      if (hda_format == NULL)
+	{
+	  fprintf(stderr, "Unknown format for disk image %s\n", hda);
+	  exit(1);
+	}
+
+      if (hdb_format == NULL)
+	{
+	  fprintf(stderr, "Unknown format for disk image %s\n", hdb);
+	  exit(1);
+	}
+
       asprintf(&hda_command,
-	       "file=%s,index=0,media=disk,cache=writeback,id=hd0",
-	       strdupa(hda));
+	       "file=%s,format=%s,index=0,media=disk,cache=writeback,id=hd0",
+	       hda, hda_format);
 
       asprintf(&hdb_command,
-	       "file=%s,index=1,media=disk,cache=writeback,id=hd1",
-	       strdupa(hdb));
+	       "file=%s,format=%s,index=1,media=disk,cache=writeback,id=hd1",
+	       hdb, hdb_format);
 
+      /* panic < 0 means reboot immediately on panic; this will actually
+         halt as -no-reboot is given to qemu */
       asprintf(&append_command,
-	       "root=/dev/%sa quiet init=/pbuilder-run console=%s",
+	       "root=/dev/%sa quiet init=/pbuilder-run console=%s panic=-1",
 	       qemu_arch_diskdevice(pc),
 	       qemu_arch_tty(pc->arch));
 
@@ -306,6 +351,7 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
       argv[argc++]=strdupa(qemu);
       argv[argc++]="-nodefaults";
       argv[argc++]="-nographic";
+      argv[argc++]="-no-reboot"; /* halt instead of rebooting */
       argv[argc++]="-M";
       argv[argc++]=strdupa(machine);
       argv[argc++]="-m";
@@ -777,9 +823,8 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
 	  //TODO: "dpkg --purge $REMOVEPACKAGES\n"
 	  //recover aptcache
 	  "apt-get -y --force-yes -o DPkg::Options::=--force-confnew dist-upgrade || exit_from_qemu 1\n"
-	  "apt-get install --force-yes -y build-essential dpkg-dev apt aptitude pbuilder || exit_from_qemu 1\n"
-	  //TODO: EXTRAPACKAGES handling
-	  //save aptcache
+	  "apt-get install --force-yes -y build-essential dpkg-dev apt aptitude pbuilder %s || exit_from_qemu 1\n"
+	  //TODO: save aptcache
 	  //optionally autoclean aptcache
 	  //run E hook
 	  //TODO: I can mount /var/cache/apt/archives from some scratch space to not need this:
@@ -789,7 +834,8 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
 	  timestring,
 	  timestring,
 	  t=sanitize_mirror(pc->mirror), pc->distribution, pc->components,
-	  pc->othermirror?pc->othermirror:"");
+	  pc->othermirror?pc->othermirror:"",
+	  pc->extrapackages?pc->extrapackages:"");
   fclose(f);
   f = NULL;
 
@@ -856,11 +902,13 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile)
   asprintf(&commandline,
 	   /* TODO: executehooks D: */
 	   "/usr/lib/pbuilder/pbuilder-satisfydepends --control $PBUILDER_MOUNTPOINT/*.dsc --internal-chrootexec 'chroot . ' %s \n"
+	   "apt-get install --force-yes -y %s\n"
 	   "cd $PBUILDER_MOUNTPOINT; /usr/bin/dpkg-source -x $(basename %s) \n"
 	   "echo ' -> Building the package'\n"
 	   /* TODO: executehooks A: */
 	   "cd $PBUILDER_MOUNTPOINT/*-*/; dpkg-buildpackage -us -uc %s\n",
 	   buildopt,
+	   pc->extrapackages?pc->extrapackages:"",
 	   dscfile,
 	   debbuildopts);
 
@@ -924,19 +972,23 @@ int cpbuilder_update(const struct pbuilderconfig* pc)
   /* TODO: --override-config support, --othermirror support etc.
      There is no way to change distribution in this code-path...
    */
-  return run_second_stage_script
-    (1,
-     //TODO: installaptlines if required.
-     //TODO: "dpkg --purge $REMOVEPACKAGES\n"
-     //TODO: add error code handling.
-     "apt-get update -o Acquire::PDiffs=false\n"
-     "apt-get -y --force-yes -o DPkg::Options::=--force-confnew dist-upgrade\n"
-     "apt-get install --force-yes -y build-essential dpkg-dev apt aptitude pbuilder\n"
-     //TODO: EXTRAPACKAGES handling
-     //optionally autoclean aptcache
-     //run E hook
-     , pc,
-     NULL, NULL);
+  char *script;
+  if (0<asprintf(&script,
+		 //TODO: installaptlines if required.
+		 //TODO: "dpkg --purge $REMOVEPACKAGES\n"
+		 //TODO: add error code handling.
+		 "apt-get update -o Acquire::PDiffs=false\n"
+		 "apt-get -y --force-yes -o DPkg::Options::=--force-confnew dist-upgrade\n"
+		 "apt-get install --force-yes -y build-essential dpkg-dev apt aptitude pbuilder %s\n"
+		 //TODO: optionally autoclean aptcache
+		 //run E hook
+		 , pc->extrapackages?pc->extrapackages:""))
+    {
+      fprintf(stderr, "qemubuilder: out of memory.\n");
+      return 1;
+    }
+
+  return run_second_stage_script(1, script, pc, NULL, NULL);
 }
 
 int cpbuilder_help(void)
